/*
 * Beyond_Arduino 8 - Low Power 2.c
 *
 *	Nuts & Volts - Beyond Arduino #8
 *  --------------------------------
 *	This project demonstrates ways to reduce power through 3 experiments.
 *  A push-button is used to toggle the LED.
 *
 *  Measure the current using a multimeter as you perform each experiment.
 *     1. Unused modules are disabled using Power Reduction Register
 *     2. The MCU is placed into sleep mode
 *     3. BOD is disabled during sleep mode
 *
 *  Connections: 
 *    - LED on PB0
 *    - Momentary Button on PB1
 *
 *  Author: Andrew Retallack
 *			www.crash-bang.com
 */ 


#include <avr/io.h>
#include <avr/interrupt.h>	//Interrupt macros
#include <avr/sleep.h>		//Sleep-related macros
#include <util/delay.h>		//Library to handle delays


//Define the LED port and pins
#define LED_PORT		PORTB
#define LED_PIN			PORTB0	//The LED that toggles
#define LED_PORT_DIR	DDRB

#define BTN_PORT		PORTB
#define BTN_PIN			PORTB1	//The momentary pushbutton
#define BTN_PORT_DIR	DDRB


int main(void)
{
	//Initialise the IO Pins
	LED_PORT_DIR |= (1<<LED_PIN);	//Output

	BTN_PORT_DIR &= ~(1<<BTN_PIN);	//Input
	BTN_PORT |= (1<<BTN_PIN);		//Enable Pull-up Resistor
	
	//Initialise Interrupt: Pin Change
	PCICR |= (1<<PCIE0);	//Enable interrupts on PCINT1 (PB1) - PCINT1 is part of group controlled by PCIE0
	PCMSK0 |= (1<<PCINT1);	//Enable interrupts on pin PCINT1 - accessed by the PCMSK0 register

	//Initialise Sleep Mode
	set_sleep_mode(SLEEP_MODE_PWR_SAVE);	//Set Sleep Mode to "Power-Save" - Timer2 can be used to wake from this mode.

//****************************************************************
//**Experiment 1: Disable unused modules**  (Uncomment line below)
//****************************************************************
//	PRR |= (1<<PRTWI) | (1<<PRTIM2) | (1<<PRTIM1) | (1<<PRTIM0) | (1<<PRUSART0) | (1<<PRSPI) | (1<<PRADC);


	sei();	//Enable Interrupts

    while(1)
    {
        
//****************************************************************
//**Experiment 3: Disable BOD during sleep**  (Uncomment line below)
//****************************************************************
//		sleep_bod_disable();	//This is a macro defined in "sleep.h"
		

//****************************************************************
//**Experiment 2: Enable Sleep Mode**  (Uncomment line below)
//****************************************************************
//		sleep_mode();	//Go to sleep


		//**This is where execution continues once the MCU has woken, and after the ISR has run.
		
		LED_PORT ^= (1<<LED_PIN);	//Toggle the LED
		_delay_ms(50);		//Delay to de-bounce the button
	
		PCICR |= (1<<PCIE0);	//Re-enable interrupts on PCINT1
		
    }
}


/***********************************************************************
*
* This is the ISR (interrupt service routine) for PCINT0
*   It is triggered on a pin-change for "bank" 0
*
************************************************************************/
ISR(PCINT0_vect)
{
	PCICR &= ~(1<<PCIE0);	//Disable interrupts on PCINT: prevents button-bounce

}
