/*
 * Beyond_Arduino 8 - 1 Low Power.c
 *
 *	Nuts & Volts - Beyond Arduino #8
 *  --------------------------------
 *	This project demonstrates the use of sleep mode.
 *	Timer2 is used to wake the MCU from sleep.  Two LEDs are connected:
 *	- LED_BLINK: this blinks at a multiplied rate of Timer2 - for every 10 interrupts on Timer2, the LED blinks.
 *  - LED_SLEEP: this blinks to prove that the MCU does enter sleep mode - it toggles before entering sleep mode.
 *	
 *
 *  Connections: 
 *    - LED_BLINK on PB0
 *    - LED_SLEEP on PB1
 *
 *  Author: Andrew Retallack
 *			www.crash-bang.com
 */ 


#include <avr/io.h>
#include <avr/interrupt.h>	//Interrupt macros
#include <avr/sleep.h>		//Sleep-related macros


//Define the LED port and pins
#define LED_PORT		PORTB
#define LED_BLINK_PIN	PORTB0	//The blinking LED is on PB0
#define LED_SLEEP_PIN	PORTB1	//The LED proving Sleep mode is on PB1
#define LED_PORT_DIR	DDRB

//Global Variables
volatile uint8_t delayCounter = 0;	//A counter used to multiply the short delay on Timer2 into a longer blink delay

//Function Prototypes
void Timer_Init(void);


int main(void)
{


	//Initialise the IO Pins
	LED_PORT_DIR |= (1<<LED_BLINK_PIN)|(1<<LED_SLEEP_PIN);	//Set pin to Output
	
	//Initialise the Timer2
	Timer_Init();

	//Initialise Sleep Mode 
	set_sleep_mode(SLEEP_MODE_PWR_SAVE);	//Set Sleep Mode to "Power-Save" - Timer2 can be used to wake from this mode.

	sei();	//Enable Interrupts

    
	while(1)
    {
		
		//LED_SLEEP proves that the MCU is actually in sleep mode, by toggling once per iteration of the WHILE loop:
		//  If the MCU goes to sleep, the LED should flash (although fast) - this proves the WHILE loop has been halted at the sleep command.
		//  If the MCU doesn't sleep, then the LED will appear to be continuously lit (actually 50% duty cycle) - as execution does not halt at the sleep command
		//Test this: by commenting out the "sleep_mode()" command and noting the difference.
		
		LED_PORT ^= (1<<LED_SLEEP_PIN);	//Toggle the LED_SLEEP
		
		sleep_mode();	//Go to sleep
		
		
		//**This is where execution continues once the MCU has woken, and after the ISR has run.
		
		
		//If a reasonable delay has elapsed, then toggle the LED_BLINK
		if (delayCounter >= 10)
		{
			LED_PORT ^= (1<<LED_BLINK_PIN);	//Toggle LED
			delayCounter = 0;  //Reset delayCounter
		}
		
		
    }


}


/***********************************************************************
*
* This function initialises Timer2 to trigger an
* interrupt when Compare occurs.
*
* INPUT:	none
* RETURNS:	none
*
************************************************************************/
void Timer_Init(void)
{

	TIMSK2 = 0;	//Disable all interrupts on timer
	
	TCCR2B |= (1<<CS22) | (1<<CS21) | (1<<CS20); //Set prescaler to 1024
	
	TCCR2A = (1<<WGM21);	//Set to Clear-Timer-on-Compare-Match (CTC) mode
		
	OCR2A = 250;	// Set Compare value: Higher value = longer delay
	
	TIMSK2 |= (1<<OCIE2A);	//Enable interrupts on OCR2A Compare Match
	
}


/***********************************************************************
*
* Interrupt handler for Timer 1: Triggers on Timer Compare
*
* INPUT:	none
* RETURNS:	none
*
************************************************************************/

ISR(TIMER2_COMPA_vect)
{

	delayCounter += 1;	//Increase the delay Counter.  Main WHILE loop will decide whether to blink the LED based on the value of delayCounter
	
	//The Timer2 interrupt has woken the MCU - execution will now continue in the main WHILE loop at the statement after the sleep command
	
}
